# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import re
from pathlib import Path

from .target import Target
from .param import PathParameter
from .task import BaseTask, Task

# Wrapper for an input file
class InputTask(BaseTask):
    src = PathParameter()

    def requires(self):
        return []

    def output(self):
        return Target(self, self.src)

    def run(self):
        if not Path(self.src).exists():
            raise OSError(1, "Input file not found", self.src)



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from contextlib import contextmanager
from pathlib import Path
from plumbum import local

from .logging import logger

# Change directory within a context
@contextmanager
def chdir(path):
    dirCurrent = Path.cwd()
    os.chdir(path)
    try:
        yield path
    finally:
        os.chdir(dirCurrent)

# Mkdir + chdir
@contextmanager
def mkcd(path):
    Path(path).mkdir(parents=True, exist_ok=True)
    with chdir(path):
        yield path

# Change environment within a context
@contextmanager
def withEnv(**kwargs):
    envOld = {key: os.environ[key] for key in kwargs if key in os.environ}
    os.environ.update(kwargs)
    try:
        with local.env(**kwargs):
            yield
    finally:
        for key, val in kwargs.items():
            if key in envOld:
                os.environ[key] = envOld[key]
            else:
                del os.environ[key]

# Wrapper of os.getenv just for convenience
def getenv(*args, **kwargs):
    return os.getenv(*args, **kwargs)

# Format a list of commands
def cmdfmt(lst, *args, **kwargs):
    lst = [s.format(*args, **kwargs) for s in lst]
    return local[lst[0]][lst[1:]]



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pickle
from base64 import b85encode, b85decode
from pathlib import Path

import luigi as lg
from luigi.task import flatten

class PathParameter(lg.PathParameter):
    """An extended type of PathParameter from the luigi one.

    This class has an additional serializeShort for displaying the task.
    If the specified path is inside the current directory, this class displays it as a relative path.
    """

    def serializeShort(self, x):
        pathForShow = Path(x)
        if pathForShow.is_absolute():
            if pathForShow.is_relative_to(Path.cwd()):
                pathForShow = pathForShow.relative_to(Path.cwd())
        return str(pathForShow)

# TODO: Test
class WhateverParameter(lg.Parameter):
    """A special type of parameter to contain anything.

    When serialize/deserialize, this class pickle all its contents.
    """

    def _warn_on_wrong_param_type(self, param_name, param_value):
        return

    def serialize(self, x):
        return str(b85encode(pickle.dumps(x)), encoding='ASCII')

    def parse(self, x):
        return pickle.loads(b85decode(bytes(x, encoding='ASCII')))

    def serializeShort(self, x):
        return self.serialize(x)

# TODO: Test
class TaskParameter(WhateverParameter):
    """A task, presumed to be pulled in as a dependency."""

    def _warn_on_wrong_param_type(self, param_name, param_value):
        pass # TODO: lg.Task?

    def serializeShort(self, x):
        aRepr = []
        for out in flatten(x.output()):
            if hasattr(out, 'pathRel'):
                aRepr.append(out.pathRel)
            elif hasattr(out, 'path'):
                aRepr.append(out.path)
            else:
                aRepr.append(repr(out))
        return ';'.join(sorted(aRepr))

# TODO: Test
class TaskListParameter(WhateverParameter):
    """A list of tasks, presumed to be pulled in as dependencies."""

    def _warn_on_wrong_param_type(self, param_name, param_value):
        pass # TODO: lg.Task?

    def serializeShort(self, xs):
        aRepr = []
        for out in flatten((x.output() for x in xs)):
            if hasattr(out, 'pathRel'):
                aRepr.append(out.pathRel)
            elif hasattr(out, 'path'):
                aRepr.append(out.path)
            else:
                aRepr.append(repr(out))
        return ';'.join(sorted(aRepr))



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path

import hypothesis.strategies as st
from hypothesis import given, example

from Eikthyr.param import PathParameter
import luigi as lg

class TaskForPath(lg.Task):
    p = PathParameter()

# Utility function to get the actual Parameter object
def getParameterObjAndVal(task, name):
    pReal = dict(task.get_params())[name]
    val = dict(task.get_param_values(task.get_params(), [], task.param_kwargs))[name]
    return pReal, val

@given(p=st.text())
def test_pathparameterShort(p):
    if len(p) <= 0: return
    t1 = TaskForPath(p)
    pReal, val = getParameterObjAndVal(t1, 'p')
    assert Path(pReal.serialize(val)) == Path(p)

    t2 = TaskForPath(Path.cwd() / p)
    pReal, val = getParameterObjAndVal(t2, 'p')
    assert Path(pReal.serializeShort(val)) == Path(p)

@given(p=st.text())
def test_pathparameterRestore(p):
    if len(p) <= 0: return
    t1 = TaskForPath(p)
    pReal, val = getParameterObjAndVal(t1, 'p')
    assert Path(pReal.serialize(pReal.parse(pReal.serialize(val)))) == Path(p)



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from datetime import timedelta
from pathlib import Path

from pytest import fixture
from .common import TestFieldForFile

from Eikthyr.task import Task
from Eikthyr.param import PathParameter, TaskParameter

# Put all luigi imports after Eikthyr to suppress annoying warnings
import luigi as lg

aSideEffects = []

class TaskA(Task):
    out = PathParameter()

    def run(self):
        aSideEffects.append('TaskA.run')
        with self.output().fpWrite() as fpw:
            fpw.write("Hello")

class TaskB(Task):
    src = TaskParameter()
    out = PathParameter()

    def run(self):
        aSideEffects.append('TaskB.run')
        with self.input().open('r') as fp:
            buf = fp.read()
        with self.output().fpWrite() as fpw:
            fpw.write(buf + ", ")

class TaskC(Task):
    src = TaskParameter()
    out = PathParameter()

    def run(self):
        aSideEffects.append('TaskC.run')
        with self.input().open('r') as fp:
            buf = fp.read()
        with self.output().fpWrite() as fpw:
            fpw.write(buf + "World!")

def getStdTaskChain():
    tA = TaskA('a.txt')
    tB = TaskB(tA, 'b.txt')
    tC = TaskC(tB, 'c.txt')
    return tA, tB, tC

def test_canTaskRun():
    aSideEffects.clear()
    with TestFieldForFile() as _:
        tA, tB, tC = getStdTaskChain()
        lg.build([tA, tB, tC,], local_scheduler=True, log_level='WARNING', workers=1)

        with open('a.txt', 'r') as fp:
            assert fp.read() == "Hello"

        with open('b.txt', 'r') as fp:
            assert fp.read() == "Hello, "

        with open('c.txt', 'r') as fp:
            assert fp.read() == "Hello, World!"

        assert aSideEffects == ['TaskA.run', 'TaskB.run', 'TaskC.run']

def test_canTaskReRunMtime1():
    aSideEffects.clear()
    with TestFieldForFile() as _:
        tA, tB, tC = getStdTaskChain()
        lg.build([tA, tB, tC,], local_scheduler=True, log_level='WARNING', workers=1)
        Path('b.txt').touch()
        lg.build([tA, tB, tC,], local_scheduler=True, log_level='WARNING', workers=1)

        assert aSideEffects == ['TaskA.run', 'TaskB.run', 'TaskC.run', 'TaskC.run']

def test_canTaskReRunMtime2():
    aSideEffects.clear()
    with TestFieldForFile() as _:
        tA, tB, tC = getStdTaskChain()
        lg.build([tA, tB, tC,], local_scheduler=True, log_level='WARNING', workers=1)
        Path('a.txt').touch()
        lg.build([tA, tB, tC,], local_scheduler=True, log_level='WARNING', workers=1)

        assert aSideEffects == ['TaskA.run', 'TaskB.run', 'TaskC.run', 'TaskB.run', 'TaskC.run']



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import time
from pathlib import Path

import hypothesis.strategies as st
from hypothesis import given, example
from .common import TestFieldForFile

from Eikthyr.target import Target, BinaryTarget

@given(content=st.text())
def test_writeTextFile(content):
    with TestFieldForFile() as _:
        tgt = Target("000")
        with tgt.fpWrite() as fpw:
            fpw.write("")
        # Overwrite with proper content
        with tgt.fpWrite() as fpw:
            fpw.write(content)

        assert Path("000").read_text().splitlines() == content.splitlines()

@given(content=st.binary())
def test_writeBinaryFile(content):
    with TestFieldForFile() as _:
        tgt = BinaryTarget("000")
        with tgt.fpWrite() as fpw:
            fpw.write(b"")
        # Overwrite with proper content
        with tgt.fpWrite() as fpw:
            fpw.write(content)

        assert Path("000").read_bytes() == content

@given(content=st.binary())
def test_writeDir(content):
    with TestFieldForFile() as _:
        tgt = Target("000")
        p = "999"
        with tgt.pathWrite() as fw:
            Path(fw).mkdir(parents=True)
            # Write proper content
            (Path(fw) / p).write_bytes(content)

        assert Path("000/999").read_bytes() == content

def test_mtime():
    with TestFieldForFile() as _:
        tgt = Target("000")
        with tgt.fpWrite() as fpw:
            fpw.write("123")
        time.sleep(0.01)  # Ensure there's a measurable time difference between writes
        mtimeInitial = os.path.getmtime(tgt.path)

        with tgt.fpWrite() as fpw:
            fpw.write("456")
        time.sleep(0.01)  # Ensure there's a measurable time difference between writes
        mtimeUpdated = os.path.getmtime(tgt.path)

        # Check mtime method matches file modification time
        assert tgt.mtime() == mtimeUpdated
        assert tgt.mtime() != mtimeInitial



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

import luigi as lg
from plumbum import local

from .logging import logger

class EnvCheck(object):
    _instance = None
    cmd = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = object.__new__(cls)
        return cls._instance

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.cmd != None:
            if isinstance(self.cmd, tuple) or isinstance(self.cmd, list) or isinstance(self.cmd, lg.ListParameter):
                if len(self.cmd)>0 and not self.cmd[0] in local:
                    self.failCmd(self.cmd[0])
            else:
                if not self.cmd in local:
                    self.failCmd(self.cmd)
            self.cmd = None # So that the check only runs once

    def failCmd(self, cmd):
        self.fail("Command '{}' not found".format(cmd))

    def fail(self, msg):
        logger.error("Environment check {} failed: {}".format(self.__class__.__name__, msg))
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import json
from contextlib import contextmanager
from hashlib import md5
from pathlib import Path
from shutil import rmtree

import luigi as lg
from luigi.local_target import LocalFileSystem

class LocalOverwriteFileSystem(LocalFileSystem):
    def rename_dont_move(self, path, dest):
        pathDest = Path(dest)
        if pathDest.is_dir():
            rmtree(dest)
            self.move(path, dest)
        else:
            pathDest.unlink(missing_ok=True)
            self.move(path, dest, raise_if_exists=False)

class Target(lg.LocalTarget):
    fs = LocalOverwriteFileSystem()

    def __init__(self, path, **kwargs):
        super().__init__(str(path), **kwargs)
        pathRel = Path(self.path)
        if pathRel.is_absolute():
            if pathRel.is_relative_to(Path.cwd()):
                pathRel = pathRel.relative_to(Path.cwd())
            else:
                pathRel = pathRel.relative_to(pathRel.root)
        self.pathRel = str(pathRel)

    def __repr__(self):
        return '{}({})'.format(self.__class__.__name__, self.path)

    def mtime(self):
        '''
        Get the last modification time of this target.
        The file/dir modification time is used here.
        '''
        return os.path.getmtime(self.path)

    @contextmanager
    def pathWrite(self):
        self.makedirs()
        with self.temporary_path() as f:
            yield f

    @contextmanager
    def fpWrite(self):
        with self.open('w') as fpw:
            yield fpw

class BinaryTarget(Target):
    def __init__(self, path, **kwargs):
        super().__init__(str(path), format=lg.format.Nop, **kwargs)



# -*- coding: utf-8 -*-

# Load dotenv before everything
from dotenv import load_dotenv as _load_dotenv, find_dotenv as _find_dotenv
_load_dotenv(_find_dotenv(usecwd=True), override=True)

# Deal with luigi's annoying deprecation warning
import warnings as _warnings
with _warnings.catch_warnings():
    _warnings.simplefilter("ignore")
    import luigi as lg

from . import param
from .param import PathParameter, WhateverParameter, TaskParameter, TaskListParameter
from luigi import Parameter, BoolParameter, IntParameter, FloatParameter, ListParameter, DictParameter

from . import target
from .target import Target, BinaryTarget

from . import cmd
from .cmd import chdir, mkcd, withEnv, cmdfmt, getenv

from . import task
from .task import BaseTask, Task
from .specialtask import InputTask

from . import envcheck
from .envcheck import EnvCheck

from . import logging
from .logging import logger

from . import run
from .run import run

from luigi import Config

# Convenient shortcuts from plumbum
from plumbum import local
cmd = local.cmd

# Convenient shortcuts from core python
from pathlib import Path
import shutil as sh



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
from datetime import timedelta

import luigi as lg
from luigi.interface import _WorkerSchedulerFactory
from luigi import worker

from .logging import logger

class _EikthyrFactory(_WorkerSchedulerFactory):
    def create_worker(self, scheduler, worker_processes, assistant=False):
        # Based on the suggestions in https://github.com/spotify/luigi/issues/2992
        return worker.Worker(scheduler=scheduler, worker_processes=worker_processes, assistant=assistant,
                check_complete_on_run=True,
                check_unfulfilled_deps=False,
                keep_alive=True,
                max_keep_alive_idle_duration=timedelta(seconds=1)
                )

def run(tasks, print_summary=True, workers=1):
    if isinstance(tasks, lg.Task):
        tasks = (tasks,)
    t0 = time.time()
    rtn = lg.build(tasks, local_scheduler=True, log_level='WARNING', detailed_summary=True,
            workers=workers, worker_scheduler_factory=_EikthyrFactory())
    if print_summary:
        logger.info("Total Time Spent: {:.3f}s".format(time.time() - t0))
        logger.debug(rtn.summary_text)
    if rtn.status != lg.LuigiStatusCode.SUCCESS and rtn.status != lg.LuigiStatusCode.SUCCESS_WITH_RETRY:
        raise RuntimeError("Luigi task run failed")
    return rtn



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path

import hypothesis.strategies as st
from hypothesis import given, example

import Eikthyr as eik
import plumbum
from plumbum import local

def test_chdir(fs):
    os.mkdir('123456')
    dirCurrent = Path.cwd()
    with eik.chdir('123456'):
        assert (dirCurrent / '123456') == Path.cwd()
    assert dirCurrent == Path.cwd()

def test_mkcd(fs):
    dirCurrent = Path.cwd()
    with eik.mkcd('123456'):
        assert (dirCurrent / '123456') == Path.cwd()
    assert dirCurrent == Path.cwd()

@given(val=st.text())
def test_withenv(val):
    if val.find('\0') != -1:
        return
    with eik.withEnv(EIKTEST_TEST00=val):
        assert eik.getenv('EIKTEST_TEST00') == val
        assert local.env['EIKTEST_TEST00'] == val
        with eik.withEnv(EIKTEST_TEST00="ABC"+val):
            assert eik.getenv('EIKTEST_TEST00') == "ABC"+val
            assert local.env['EIKTEST_TEST00'] == "ABC"+val

def test_cmdfmt():
    assert eik.cmdfmt(['ls', '{}/'], 'tests')



# -*- coding: utf-8 -*-
# Copyright 2021-2023, Hojin Koh
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
import pickle
from inspect import isgenerator
from pathlib import Path

import luigi as lg
from luigi.task import flatten
from colorama import Fore, Style
from plumbum import FG

from .cmd import withEnv
from .target import Target, BinaryTarget
from .logging import logger
from .param import TaskParameter, TaskListParameter

def getAllInputTargets(aTask):
    if len(aTask) == 0: return set()
    setRslt = set()
    for t in aTask:
        setRslt |= set(flatten(t.input()))
        setRslt |= getAllInputTargets(t._requires())
    return setRslt

class BaseTask(lg.Task):
    prev = TaskListParameter((), significant=False, positional=False)
    logger = logger

    #def __init__(self, *args, **kwargs):
    #    super().__init__(*args, **kwargs)
    #    self.objOutput = None

    # Mostly copied from the original luigi
    def __repr__(self):
        params = self.get_params()
        param_values = self.get_param_values(params, [], self.param_kwargs)

        # Build up task id
        repr_parts = []
        param_objs = dict(params)
        for param_name, param_value in param_values:
            if param_objs[param_name].significant:
                # If there is serializeShort we added, use it
                if hasattr(param_objs[param_name], 'serializeShort'):
                    thisRepr = param_objs[param_name].serializeShort(param_value)
                else:
                    thisRepr = param_objs[param_name].serialize(param_value)
                repr_parts.append('{}={}'.format(param_name, thisRepr))

        return '{}({})'.format(self.get_task_family(), ', '.join(repr_parts))

    def _requires(self):
        return flatten(self.requires()) + list(self.prev)

    # The default input if there's a parameter called "src"
    def requires(self):
        if hasattr(self, 'src'):
            return self.src
        else:
            return []

    # The default output if there's a parameter called "out" or "outbin"
    def output(self):
        if hasattr(self, 'out'):
            return Target(self.out)
        elif hasattr(self, 'outbin'):
            return BinaryTarget(self.outbin)
        else:
            return []

    def complete(self):
        """
        Return `True` if all output files exist and their modification time is newer than
        the modification time of any input file. Otherwise, return `False`.
        """
        aInputs = list(getAllInputTargets([self]))
        aOutputs = flatten(self.output())

        # First, still check the output exists, as in luigi
        if not all((output.exists() for output in aOutputs)):
            return False

        # Collect the list of mtime from outputs. If none can be checked, then just assume they're okay
        aMtimesOutput = [output.mtime() for output in aOutputs if hasattr(output, 'mtime')]
        if len(aMtimesOutput) == 0:
            return True

        # Collect the list of mtime from inputs. If none can be checked, then just assume they're okay
        aMtimesInput = [obj.mtime() for obj in aInputs if hasattr(obj, 'mtime')]
        if len(aMtimesInput) == 0:
            return True

        # Reaching here, all outputs exist, and both input side and output side has some mtime for comparison
        if max(aMtimesInput) > min(aMtimesOutput):
            return False

        return True

    # Expected to get a plumbum object
    def ex(self, chain):
        self.logger.info("RUN: {}".format(chain))
        chain & FG

class Task(BaseTask):
    pass

@Task.event_handler(lg.Event.START)
def logTaskStart(task):
    logger.debug("{}{}Start {}{}".format(Fore.CYAN, Style.BRIGHT, task, Style.RESET_ALL))

@Task.event_handler(lg.Event.PROCESSING_TIME)
def logTaskStart(task, t):
    logger.info("{}{}Done {} in {:.1f}s{}\n".format(Fore.GREEN, Style.BRIGHT, task, t, Style.RESET_ALL))
